/*****************************************************************
 * Copyright (c) 2018 Texas Instruments and others
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *  Paul Gingrich - Initial API and implementation
 *****************************************************************/
var gc = gc || {};
gc.databind = gc.databind || {};
gc.databind.internal = gc.databind.internal || {};

(function() 
{
    gc.databind.internal.GenericFrameDecoder = function(startByte, getLengthMethod)
    {
        this.isConnected = false;
        this.partialPacket = [];
        this.frameStart = startByte;
        this.getpacketLength = getLengthMethod;
    };

    gc.databind.internal.GenericFrameDecoder.prototype = new gc.databind.IPacketCodec();

    gc.databind.internal.GenericFrameDecoder.prototype.encode = function(target, value) 
    {
        // convert to string message
        var message = value;
        if (value instanceof Array)
        {
            message = String.fromCharCode.apply(null, value);
        }
        target(message);
    };

    gc.databind.internal.GenericFrameDecoder.prototype.decode = function(target, rawData)
    {
        if (this._partialPacket)
        {
            // concatentate new data with saved partial data
            rawData = this._partialPacket.concat(rawData);
            this._partialPacket = null; 
        }
        var i = 0;
        var length = rawData.length;
        
        // process one or more packets (whether valid of not).
        while(i < length)
        {   
            // if provided, test for the start of packet.
            if (this.frameStart)
            {
                // skip till first delimiter
                while(i < length && rawData[i] !== this.frameStart)
                {
                    i++;
                    this.isConnected = false; // discarding data.
                }
            }
            
            if (i < length)
            {
                // calculate the length of the packet, zero if not enough data to determine, and -1 for error packets  
                var packetLength = this.getpacketLength(rawData, i);
                if (packetLength > 0 && packetLength + i <= length)
                {
                    // enough data to decode a packet.
                    this.isConnected = target((i === 0 && length === packetLength) ? rawData : rawData.slice(i, i + packetLength));
                    if (this.isConnected)
                    {
                        // valid packet processed, skip forward
                        i += packetLength;
                    }
                    else  // invalid packet decoded, skip forward only by one byte to search for start of valid packet.
                    { 
                        i++;
                    }
                }
                else if (packetLength < 0)
                {
                    // invalid packet already, skip forward one or more bytes.
                    i -= packetLength;
                    this.isConnected = false; // discarding data.
                }
                else if (i === 0)
                {
                    // no data consumed, save all data for next pass.
                    this._partialPacket = rawData; 
                    break;
                }
                else if (i < length)
                {
                    // not enough data remaining for a single packet, defer the partial packet until more data has arrived.
                    this._partialPacket = rawData.slice(i);
                    break;
                }
            }
        }
        return this.isConnected;
    };
}());
