var gc = gc || {};


/*
Packet OCF_PACKET_HEADER:
{

    uint16_t     signature;     // Packet signature (must be PACKET_SIGNATURE value)
    uint16_t     type;          // Type of packet
    uint32_t     transfer_len;  // Total number of bytes in the transfer
    uint16_t     packet_num;    // Sequential packet number
    OC_STATUS    status;        // Status code returned by command function
    uint16_t     payload_len;   // Number of bytes in the payload
    uint16_t     if_type_unit;  // Interface type and unit
    uint16_t     command;       // Command code
    uint16_t     reserved;      // unused (for 32-bit alignment)
    uint32_t     param[8];      // eight 32-bit parameters
}
*/

(function() {
    /* set true to turn on logging to the console of all packets sent and received. */
    var LOG_PACKETS  = true;

    /*********************************************************************************************************************
     * 
     * USB2ANY2
     * 
     *********************************************************************************************************************/
    
    var USB2ANY2 = function()  {
        this.packetParser = new RegisterPacketParser();
        this.ocfIF = {};
    }
    USB2ANY2.prototype = new gc.databind.IPacketCodec();

    USB2ANY2.prototype.connect = function(settings) {
        return Q.Promise(function(resolve, reject) {
            var parser = this.packetParser;
            var sendPacket = this.encoder.bind(this);

            this.ocfIF.system = new OCFSystemInterface(parser, sendPacket);
            this.ocfIF.system.init(settings).then(function() {
                var ifPromises = [];

                var devices = settings.all.devices;
                for (var i = 0; i < devices.length; ++i) {
                    switch(devices[i].interface) {    
                        case 'i2c':
                            this.ocfIF.i2c = devices[i]._if = new OCFI2CInterface(parser, sendPacket);
                            ifPromises.push(this.ocfIF.i2c.init(settings));
                            break;
        
                        case 'gpio':
                            break;
                    }
                }
                return Q.all(ifPromises).then(resolve);
            
            }.bind(this)).fail(function(err) {
                reject(err);
            });;
        }.bind(this));
    };

    USB2ANY2.prototype.disconnect = function(callback) {
        this.reset();
    };

    USB2ANY2.prototype.readValue = function(registerInfo) {
        var device = registerInfo.parentGroup.parentDevice;
        if (device._if != null) {
            return device._if.read(registerInfo);
        } else {
            return Q.reject('Cannot read value, undefined ' + device.interface + ' OCF interface.');
        }
    };

    USB2ANY2.prototype.writeValue = function(registerInfo, value) {
        var device = registerInfo.parentGroup.parentDevice;
        if (device._if != null) {
            return device._if.read(registerInfo, value);
        } else {
            return Q.reject('Cannot write value, undefined ' + device.interface + ' OCF interface.');
        }
    };

    USB2ANY2.prototype.encode = function(target, value) {
        var packet = this.packetParser.encode(value);
        if (LOG_PACKETS)
            gc.console.log('USB2ANY2', 'encode: ' + packet);

        target(packet);
    };

    USB2ANY2.prototype.decode = function(target, rawData) {
        if (LOG_PACKETS)
            gc.console.log('USB2ANY2', 'decode: ' + rawData);

        var handled = false;
        var parser = this.packetParser;
        var packet = parser.decode(rawData);
        var interface = parser.getPacketInterface(packet);
        try {
            switch (interface) {
                case 0x00: // system
                    if (typeof this.ocfIF.system != 'undefined') {
                        handled = this.ocfIF.system.handlePacket(packet);
                    }
                    break;

                case 0x01: // memory
                    break;

                case 0x02: // port
                    break;

                case 0x03: // i2c
                    if (typeof this.ocfIF.i2c != 'undefined') {
                        handled = this.ocfIF.i2c.handlePacket(packet);
                    }
                    break;

                case 0x04: // gpio
                    break;
            }
        } catch (err) {
            gc.console.error('USB2ANY2', err);
        }
        return handled;
    };

    USB2ANY2.prototype.reset = function() {
        for (var property in this.ocfIF) {
            if (this.ocfIF.hasOwnProperty(property)) {
                this.ocfIF[property].reset();
            }
        }
    };

    var getPacketLength = function(buffer, offset) {
        // TODO:
        return 0;
        //return buffer.length - offset < PACKET_HEADER_SIZE ? 0 : buffer[offset + PACKET_PAYLOAD_LEN] + PACKET_HEADER_SIZE;
    };

    // create a frame decoder for serial USB communication where the frames are not automatically aligned.
    //var USB2ANY2_FrameDecoder = function() {
    //};
    //USB2ANY2_FrameDecoder.prototype = new gc.databind.internal.GenericFrameDecoder(PACKET_IDENTIFIER, getPacketLength);

    // register USB2ANY2 packet codec with optional frame decoder (for use with USB transport, that is not HID).
    gc.databind.registerCustomCodec('USB2ANY2', USB2ANY2, null, null /*USB2ANY2_FrameDecoder*/);

}());
